/*
 * object.h
 * Object representation.
 *
 * Copyright (c) 1996 Systems Architecture Research Centre,
 *		   City University, London, UK.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by Tim Wilkinson <tim@sarc.city.ac.uk>, February 1996.
 */

#ifndef __object_h
#define __object_h

#include "gtypes.h"
#include "locks.h"

struct _methodTable;
struct _classes;

typedef struct _object {
	struct _methodTable*	mtable;
	uint32			idx;
	uint32			size;
	iMux			mux;
	iCv			cv;
	/* Data follows on immediately */
} object;

/*
 * These bizzare casts provide various offset into the object structure.
 */
#define	OBJECT_MTABLE		((int)&(*(object*)0).mtable)
#define	OBJECT_IDX		((int)&(*(object*)0).idx)
#define	OBJECT_SIZE		((int)&(*(object*)0).size)
#define	OBJECT_DATA		((int)(((object*)0)+1))

object*			alloc_object(struct _classes*, bool);
struct _classes*	alloc_class(void);
object*			alloc_array(int, int);
object*			alloc_objectarray(int, char*);
object*			alloc_multiarray(int*, char*);

#endif
